@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -77,6 +77,13 @@ static void offlinemode_changed(struct connman_profile *profile)
 	connman_dbus_property_changed_basic(profile->path,
 				CONNMAN_PROFILE_INTERFACE, "OfflineMode",
 				DBUS_TYPE_BOOLEAN, &profile->offlinemode);
+
+	if (profile != default_profile)
+		return;
+
+	connman_dbus_property_changed_basic(CONNMAN_MANAGER_PATH,
+				CONNMAN_MANAGER_INTERFACE, "OfflineMode",
+				DBUS_TYPE_BOOLEAN, &profile->offlinemode);
 }
 
 connman_bool_t __connman_profile_get_offlinemode(void)
