@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -23,6 +23,7 @@
 #include <config.h>
 #endif
 
+#include <stdio.h>
 #include <unistd.h>
 #include <string.h>
 #include <sys/socket.h>
@@ -240,17 +241,32 @@ static const char *operstate2str(unsigned char operstate)
 }
 
 static void extract_link(struct ifinfomsg *msg, int bytes,
-				const char **ifname, unsigned char *operstate)
+				struct ether_addr *address, const char **ifname,
+				unsigned int *mtu, unsigned char *operstate,
+						struct rtnl_link_stats *stats)
 {
 	struct rtattr *attr;
 
 	for (attr = IFLA_RTA(msg); RTA_OK(attr, bytes);
 					attr = RTA_NEXT(attr, bytes)) {
 		switch (attr->rta_type) {
+		case IFLA_ADDRESS:
+			if (address != NULL)
+				memcpy(address, RTA_DATA(attr), ETH_ALEN);
+			break;
 		case IFLA_IFNAME:
 			if (ifname != NULL)
 				*ifname = RTA_DATA(attr);
 			break;
+		case IFLA_MTU:
+			if (mtu != NULL)
+				*mtu = *((unsigned int *) RTA_DATA(attr));
+			break;
+		case IFLA_STATS:
+			if (stats != NULL)
+				memcpy(stats, RTA_DATA(attr),
+					sizeof(struct rtnl_link_stats));
+			break;
 		case IFLA_OPERSTATE:
 			if (operstate != NULL)
 				*operstate = *((unsigned char *) RTA_DATA(attr));
@@ -264,22 +280,41 @@ static void extract_link(struct ifinfomsg *msg, int bytes,
 static void process_newlink(unsigned short type, int index, unsigned flags,
 			unsigned change, struct ifinfomsg *msg, int bytes)
 {
+	struct ether_addr address = {{ 0, 0, 0, 0, 0, 0 }};
+	struct ether_addr compare = {{ 0, 0, 0, 0, 0, 0 }};
+	struct rtnl_link_stats stats;
 	unsigned char operstate = 0xff;
 	const char *ifname = NULL;
+	unsigned int mtu = 0;
+	char str[18];
 	GSList *list;
 
+	memset(&stats, 0, sizeof(stats));
+	extract_link(msg, bytes, &address, &ifname, &mtu, &operstate, &stats);
+
+	snprintf(str, 18, "%02X:%02X:%02X:%02X:%02X:%02X",
+						address.ether_addr_octet[0],
+						address.ether_addr_octet[1],
+						address.ether_addr_octet[2],
+						address.ether_addr_octet[3],
+						address.ether_addr_octet[4],
+						address.ether_addr_octet[5]);
+
 	switch (type) {
 	case ARPHRD_ETHER:
 	case ARPHRD_LOOPBACK:
 	case ARPHRD_NONE:
-		__connman_ipconfig_newlink(index, type, flags);
+		__connman_ipconfig_newlink(index, type, flags,
+							str, mtu, &stats);
 		break;
 	}
 
-	extract_link(msg, bytes, &ifname, &operstate);
+	if (memcmp(&address, &compare, ETH_ALEN) != 0)
+		connman_info("%s {newlink} index %d address %s mtu %u",
+						ifname, index, str, mtu);
 
 	if (operstate != 0xff)
-		connman_info("%s {newlink} index %d operstate %d <%s>",
+		connman_info("%s {newlink} index %d operstate %u <%s>",
 						ifname, index, operstate,
 						operstate2str(operstate));
 
@@ -307,14 +342,16 @@ static void process_newlink(unsigned short type, int index, unsigned flags,
 static void process_dellink(unsigned short type, int index, unsigned flags,
 			unsigned change, struct ifinfomsg *msg, int bytes)
 {
+	struct rtnl_link_stats stats;
 	unsigned char operstate = 0xff;
 	const char *ifname = NULL;
 	GSList *list;
 
-	extract_link(msg, bytes, &ifname, &operstate);
+	memset(&stats, 0, sizeof(stats));
+	extract_link(msg, bytes, NULL, &ifname, NULL, &operstate, &stats);
 
 	if (operstate != 0xff)
-		connman_info("%s {dellink} index %d operstate %d <%s>",
+		connman_info("%s {dellink} index %d operstate %u <%s>",
 						ifname, index, operstate,
 						operstate2str(operstate));
 
@@ -339,7 +376,7 @@ static void process_dellink(unsigned short type, int index, unsigned flags,
 	case ARPHRD_ETHER:
 	case ARPHRD_LOOPBACK:
 	case ARPHRD_NONE:
-		__connman_ipconfig_dellink(index);
+		__connman_ipconfig_dellink(index, &stats);
 		break;
 	}
 }
@@ -746,15 +783,29 @@ static void rtnl_route(struct nlmsghdr *hdr)
 	}
 }
 
+static connman_bool_t is_route_rtmsg(struct rtmsg *msg)
+{
+
+	if (msg->rtm_table != RT_TABLE_MAIN)
+		return FALSE;
+
+	if (msg->rtm_protocol != RTPROT_BOOT &&
+			msg->rtm_protocol != RTPROT_KERNEL)
+		return FALSE;
+
+	if (msg->rtm_type != RTN_UNICAST)
+		return FALSE;
+
+	return TRUE;
+}
+
 static void rtnl_newroute(struct nlmsghdr *hdr)
 {
 	struct rtmsg *msg = (struct rtmsg *) NLMSG_DATA(hdr);
 
 	rtnl_route(hdr);
 
-	if (msg->rtm_table == RT_TABLE_MAIN &&
-				msg->rtm_protocol == RTPROT_BOOT &&
-						msg->rtm_type == RTN_UNICAST)
+	if (is_route_rtmsg(msg))
 		process_newroute(msg->rtm_family, msg->rtm_scope,
 						msg, RTM_PAYLOAD(hdr));
 }
@@ -765,9 +816,7 @@ static void rtnl_delroute(struct nlmsghdr *hdr)
 
 	rtnl_route(hdr);
 
-	if (msg->rtm_table == RT_TABLE_MAIN &&
-				msg->rtm_protocol == RTPROT_BOOT &&
-						msg->rtm_type == RTN_UNICAST)
+	if (is_route_rtmsg(msg))
 		process_delroute(msg->rtm_family, msg->rtm_scope,
 						msg, RTM_PAYLOAD(hdr));
 }
@@ -1014,6 +1063,11 @@ static int send_getroute(void)
 	return queue_request(req);
 }
 
+int __connman_rtnl_request_update(void)
+{
+	return send_getlink();
+}
+
 int __connman_rtnl_init(void)
 {
 	struct sockaddr_nl addr;
